@@ -22,6 +22,7 @@
 
 #include "sample.h"
 #include "loop.h"
+//#include "modal.h"
 
 static char *decapitalize (char *s)
 {
@@ -30,6 +31,18 @@ static char *decapitalize (char *s)
     return s;
 }
 
+static int dummy_index(lua_State *L)
+{
+    lua_pushnil (L);
+    
+    return 1;
+}
+
+static int dummy_newindex(lua_State *L)
+{
+    return 0;
+}
+
 static int generic_tostring(lua_State *L)
 {
     id object;
@@ -144,6 +157,19 @@ static int sample_call (lua_State *L)
     return 1;
 }
 
+static int sample_tostring(lua_State *L)
+{
+    id object;
+
+    object = *(id *)lua_touserdata(L, 1);
+
+    lua_pushstring(L, [object name]);
+    lua_pushstring(L, " foundry");
+    lua_concat(L, 2);
+    
+    return 1;
+}
+
 static int sample_gc(lua_State *L)
 {
     id object;
@@ -223,7 +249,7 @@ int constructsample(lua_State *L)
 
     lua_newtable(L);
     lua_pushstring(L, "__tostring");
-    lua_pushcfunction(L, generic_tostring);
+    lua_pushcfunction(L, sample_tostring);
     lua_settable(L, -3);
     lua_pushstring(L, "__call");
     lua_pushcfunction(L, sample_call);
@@ -231,26 +257,102 @@ int constructsample(lua_State *L)
     lua_pushstring(L, "__gc");
     lua_pushcfunction(L, sample_gc);
     lua_settable(L, -3);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, dummy_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, dummy_newindex);
+    lua_settable(L, -3);
     lua_setmetatable(L, -2);
 
+    lua_pushstring (L, "userdata");
+    lua_gettable (L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (L, object);
+    lua_pushvalue (L, -3);
+    lua_settable (L, -3);
+    lua_pop(L, 1);
+
     free (samples);
     
     return 1;
 }
 
+static int constructnode (lua_State *L)
+{
+    Class class;
+    id object;
+
+    lua_pushvalue (L, lua_upvalueindex (1));
+    class = (Class)lua_touserdata(L, -1);
+    lua_pop(L, 1);
+
+    /* Create the userdata... */
+
+    object = [[class alloc] init];
+    *(id *)lua_newuserdata(L, sizeof(id)) = object;
+
+    lua_newtable (L);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, (lua_CFunction)generic_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, (lua_CFunction)generic_newindex);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__tostring");
+    lua_pushcfunction(L, (lua_CFunction)generic_tostring);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__gc");
+    lua_pushcfunction(L, (lua_CFunction)generic_gc);
+    lua_settable(L, -3);
+    lua_setmetatable(L, -2);
+
+    /* Add a weak reference. */
+    
+    lua_pushstring (L, "userdata");
+    lua_gettable (L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (L, object);
+    lua_pushvalue (L, 2);
+    lua_settable (L, -3);
+    lua_pop(L, 1);
+    
+    /* ...and initialize it. */
+
+    if(lua_istable(L, 1)) {
+	lua_pushnil(L);
+	
+	while(lua_next(L, 1)) {
+	    lua_pushvalue(L, -2);
+	    lua_insert(L, -2);
+	    lua_settable(L, 2);
+	}
+    }
+
+    return 1;
+}
+
 int luaopen_accoustics (lua_State *L)
 {
     int i;
 
-    Class nodes[] = {
+    Class samples[] = {
 	[Sample class], [Loop class]
     };	
 
+    Class nodes[] = {
+//	[Modal class]
+    };	
+
     lua_newtable (L);    
     
+    for (i = 0 ; i < sizeof(samples) / sizeof(samples[0]) ; i += 1) {
+	lua_pushlightuserdata (L, samples[i]);
+	lua_pushcclosure (L, constructsample, 1);
+	lua_setfield(L, -2, decapitalize(strdupa([samples[i] name])));
+    }
+  
     for (i = 0 ; i < sizeof(nodes) / sizeof(nodes[0]) ; i += 1) {
 	lua_pushlightuserdata (L, nodes[i]);
-	lua_pushcclosure (L, constructsample, 1);
+	lua_pushcclosure (L, constructnode, 1);
 	lua_setfield(L, -2, decapitalize(strdupa([nodes[i] name])));
     }
 
