@@ -18,16 +18,16 @@
 #define _AMBIENT_H_
 
 #include <GL/gl.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Ambient: Node {    
-    GLfloat frame[9], intensityConstant[3];
+@interface Ambient: Transform {    
+    GLfloat intensityConstant[3];
     id intensityMap;
 }
 
--(GLfloat *)frame;
--(GLfloat *) intensityConstant;
--(GLuint) intensityMap;
+-(void) loadOrientationInto: (unsigned int)that;
+-(void) loadIntesityConstantInto: (unsigned int)this
+                      andMapInto: (unsigned int)that;
 
 @end
 
