@@ -30,7 +30,7 @@
     };
 
     self->joint = dJointCreateAMotor (_WORLD, NULL);
-
+    
     self->axes[0][0] = 1;
     self->axes[0][1] = 0;
     self->axes[0][2] = 0;
@@ -58,6 +58,28 @@
     return self;
 }
 
+-(void) setup
+{
+    int i;
+
+    for (i = 0 ; i < self->degrees ; i += 1) {
+	dJointSetAMotorAxis ([self joint], i,
+			     dJointGetBody ([self joint], 0) ? 1 : 0,
+			     self->axes[i][0],
+			     self->axes[i][1],
+			     self->axes[i][2]);
+    }
+}
+
+-(void) update
+{
+    /* Reset the axes after reattaching. */
+
+    [super update];
+    [self setup];
+}
+
+
 -(void) get
 {
     const char *k;
@@ -72,7 +94,7 @@
 
 	lua_newtable (_L);
         
-	for(j = 0 ; j < 3 ; j += 1) {
+	for(j = 0 ; j < self->degrees ; j += 1) {
 	    lua_newtable (_L);
         
 	    for(i = 0; i < 3; i += 1) {
@@ -85,7 +107,7 @@
     } else if (!xstrcmp(k, "motor")) {
 	lua_newtable (_L);
         
-	for(j = 0 ; j < 3 ; j += 1) {
+	for(j = 0 ; j < self->degrees ; j += 1) {
 	    lua_newtable (_L);
         
 	    for(i = 0; i < 2; i += 1) {
@@ -98,7 +120,7 @@
     } else if (!xstrcmp(k, "stops")) {
 	lua_newtable (_L);
         
-	for(j = 0 ; j < 3 ; j += 1) {
+	for(j = 0 ; j < self->degrees ; j += 1) {
 	    lua_newtable (_L);
 
 	    lua_newtable (_L);
@@ -123,14 +145,14 @@
     } else if (!xstrcmp(k, "tolerance")) {
 	lua_newtable (_L);
         
-	for(j = 0 ; j < 3 ; j += 1) {
+	for(j = 0 ; j < self->degrees ; j += 1) {
 	    lua_pushnumber (_L, self->tolerance[j]);
 
 	    lua_rawseti (_L, -2, j + 1);
 	}
     } else if (!xstrcmp(k, "state")) {
 	lua_newtable (_L);
-        
+
 	for(j = 0 ; j < self->degrees ; j += 1) {
 	    lua_pushnumber (_L, dJointGetAMotorAngle ([self joint], j));
 	    lua_rawseti (_L, -2, j + 1);
@@ -168,15 +190,14 @@
 		    }
 
 		    dSafeNormalize3 (self->axes[j]);
-		    dJointSetAMotorAxis([self joint], j,
-					dJointGetBody([self joint], 0) ? 1 : 0,
-					self->axes[j][0],
-					self->axes[j][1],
-					self->axes[j][2]);
 		}
 
 		lua_pop (_L, 1);
 	    }
+
+	    /* Setup joint axes. */
+		
+	    [self setup];
 	}
     } else if (!xstrcmp(k, "motor")) {
         if(lua_istable (_L, 3)) {
@@ -282,16 +303,6 @@
 				      self->tolerance[j]);
             }
         }
-    } else if (!xstrcmp(k, "bodies")) {
-	[super set];
-
-	for (i = 0 ; i < self->degrees ; i += 1) {
-	    dJointSetAMotorAxis ([self joint], i,
-				 dJointGetBody ([self joint], 0) ? 1 : 0,
-				 self->axes[i][0],
-				 self->axes[i][1],
-				 self->axes[i][2]);
-	}
     } else {
 	[super set];
     }
