@@ -62,7 +62,6 @@ static void callhooks (lua_State *L, void *key, int reference, int index)
     [super init];
     [self add: sizeof (list) / sizeof (char *) Properties: list];
 
-    self->mold = LUA_REFNIL;    
     self->spawn = LUA_REFNIL;    
     self->size = 0;
     
@@ -73,29 +72,43 @@ static void callhooks (lua_State *L, void *key, int reference, int index)
 {
     int i;
 
-    if (self->mold != LUA_REFNIL && n > m) {
-	/* Get a hold of yourself. */
+    /* Get a hold of yourself. */
     
-	lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
-	lua_pushlightuserdata (_L, self);
-	lua_gettable (_L, -2);
+    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
+    lua_pushlightuserdata (_L, self);
+    lua_rawget (_L, -2);
 
-	/* And start spawning. */
+    /* And of the mold. */
+    
+    lua_getmetatable (_L, -1);
+    lua_pushstring (_L, "__mold");
+    lua_gettable (_L, -2);
+    lua_replace (_L, -2);
+
+    if (!lua_isnil (_L, -1) && n > m) {
+    	/* And start spawning. */
 	
-	for (i = m ; i < n ; i += 1) {
-	    lua_pushnumber (_L, i + 1);
-	    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->mold);
-	    lua_newtable (_L);
-	    luaX_call (_L, 1, 1);
-	    lua_settable (_L, -3);
+    	for (i = m ; i < n ; i += 1) {
+    	    lua_pushnumber (_L, i + 1);
+    	    lua_pushvalue (_L, -2);
+    	    lua_newtable (_L);
+    	    luaX_call (_L, 1, 1);
+    	    lua_settable (_L, -4);
 
-	    if (self->spawn != LUA_REFNIL) {
-		callhooks (_L, self, self->spawn, i + 1);
-	    }
-	}
-
-	lua_pop (_L, 2);
+    	    if (self->spawn != LUA_REFNIL) {
+    		callhooks (_L, self, self->spawn, i + 1);
+    	    }
+    	}
     }
+
+    lua_pop (_L, 3);
+}
+
+-(void) free
+{
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->spawn);
+
+    [super free];
 }
 
 -(void) get
@@ -109,7 +122,11 @@ static void callhooks (lua_State *L, void *key, int reference, int index)
     } else if (!xstrcmp(k, "spawn")) {
         lua_rawgeti (_L, LUA_REGISTRYINDEX, self->spawn);
     } else if (!xstrcmp(k, "mold")) {
-        lua_rawgeti (_L, LUA_REGISTRYINDEX, self->mold);
+	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__mold");
+	lua_replace (_L, 2);
+        lua_rawget (_L, 1);
     } else  {
 	[super get];
     }
@@ -130,8 +147,11 @@ static void callhooks (lua_State *L, void *key, int reference, int index)
 	luaL_unref (_L, LUA_REGISTRYINDEX, self->spawn);
         self->spawn = luaL_ref (_L, LUA_REGISTRYINDEX);
     } else if (!xstrcmp(k, "mold")) {
-	luaL_unref (_L, LUA_REGISTRYINDEX, self->mold);
-        self->mold = luaL_ref (_L, LUA_REGISTRYINDEX);
+	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__mold");
+	lua_replace (_L, 2);
+        lua_rawset (_L, 1);
 
         [self spawn: 0 through: self->size];
     } else {
