@@ -67,8 +67,7 @@ static void callpoststephooks (dBodyID body)
 -(Body *) init
 {
     char *list[] = {
-	"classification", "damping", "lethargy", "mass", "orientation",
-	"position", "spin", "velocity"
+	"classification", "damping", "lethargy", "mass", "spin", "velocity"
     };
 
     /* Get the configuration. */
@@ -125,6 +124,10 @@ static void callpoststephooks (dBodyID body)
     if (self->body) {
 	dBodyDestroy (self->body);
     }
+
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->poststep);
+
+    [super free];
 }
 
 -(dBodyID) body
@@ -332,6 +335,8 @@ static void callpoststephooks (dBodyID body)
             lua_rawseti (_L, -2, i + 1);
         }
     } else if (!xstrcmp(k, "mass")) {
+      dBodyGetMass (self->body, &self->mass);
+
         lua_newtable (_L);
 
         lua_pushnumber (_L, self->mass.mass);
@@ -358,22 +363,26 @@ static void callpoststephooks (dBodyID body)
         lua_rawseti (_L, -2, 3);
     } else if (!xstrcmp(k, "classification")) {
 	unsigned int i, mask;
-	
-	mask = dGeomGetCategoryBits (self->geom);
-	lua_newtable (_L);
-	
-	for (i = 0 ; i < 32 ; i += 1) {
-	    if (mask & ((unsigned int)0x1 << i)) {
-		int n;
 
-		n = luaX_objlen (_L, -1);
-		lua_pushnumber (_L, i + 1);
-		lua_rawseti (_L, -2, n + 1);
+	if (self->geom) {
+	    mask = dGeomGetCategoryBits (self->geom);
+	    lua_newtable (_L);
+	
+	    for (i = 0 ; i < 32 ; i += 1) {
+		if (mask & ((unsigned int)0x1 << i)) {
+		    int n;
+
+		    n = luaX_objlen (_L, -1);
+		    lua_pushnumber (_L, i + 1);
+		    lua_rawseti (_L, -2, n + 1);
+		}
 	    }
-	}
 
-	if (luaX_objlen (_L, -1) == 1) {
-	    lua_rawgeti (_L, -1, 1);
+	    if (luaX_objlen (_L, -1) == 1) {
+		lua_rawgeti (_L, -1, 1);
+	    }
+	} else {
+	    lua_pushnil(_L);
 	}
     } else if (!xstrcmp(k, "lethargy")) {
 	if (self->body && dBodyGetAutoDisableFlag (self->body)) {
@@ -411,6 +420,12 @@ static void callpoststephooks (dBodyID body)
 	} else {
 	    lua_pushboolean (_L, 0);
 	}
+    } else if (!xstrcmp(k, "kinematic")) {
+	if (self->body) {
+	    lua_pushboolean (_L, dBodyIsKinematic (self->body));
+	} else {
+	    lua_pushboolean (_L, 0);
+	}
     } else if (!xstrcmp(k, "poststep")) {
         lua_rawgeti (_L, LUA_REGISTRYINDEX, self->poststep);
     } else {
@@ -571,6 +586,14 @@ static void callpoststephooks (dBodyID body)
 
 	    lua_pop (_L, 4);
 	}
+    } else if (!xstrcmp(k, "kinematic")) {
+	if (self->body) {
+	    if (lua_toboolean (_L, 3)) {
+		dBodySetKinematic (self->body);
+	    } else {
+		dBodySetDynamic (self->body);
+	    }
+	}
     } else if (!xstrcmp(k, "poststep")) {
         luaL_unref (_L, LUA_REGISTRYINDEX, self->poststep);
         self->poststep = luaL_ref (_L, LUA_REGISTRYINDEX);
