@@ -19,9 +19,9 @@
 
 #include <lua.h>
 #include <ode/ode.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Body: Node {
+@interface Body: Transform {
     dBodyID body;
     dGeomID geom;
     dSpaceID space;
@@ -29,7 +29,9 @@
     dReal velocity[3], spin[3];
     dMass mass;
 
-    int clamped, encapsulated;
+    int clamped, encapsulated, debug;
+@public    
+    int poststep;
 }
 
 -(dBodyID) body;
