@@ -20,11 +20,14 @@
 #include <assert.h>
 #include <lua.h>
 #include <ode/ode.h>
+#include <gdk/gdk.h>
+#include <gdk/gdkgl.h>
 
 lua_State *_L;
 dWorldID _WORLD;
 dSpaceID _SPACE;
 dJointGroupID _GROUP;
+GdkWindow *_WINDOW;
 
 /* #define DRAW_VOLUMES */
 
@@ -36,5 +39,17 @@ dJointGroupID _GROUP;
     }
 
 int xstrcmp (const char *a, const char *b);
+int luaX_objlen (lua_State *L, int i);
+void luaX_call (lua_State *L, int nargs, int nresults);
+void luaX_pushunsignedbytes (lua_State *L);
+void luaX_pushbytes (lua_State *L);
+void luaX_pushunsignedchars (lua_State *L);
+void luaX_pushchars (lua_State *L);
+void luaX_pushunsignedshorts (lua_State *L);
+void luaX_pushshorts (lua_State *L);
+void luaX_pushunsignedints (lua_State *L);
+void luaX_pushints (lua_State *L);
+void luaX_pushfloats (lua_State *L);
+void luaX_pushdoubles (lua_State *L);
 
 #endif
