@@ -37,8 +37,12 @@
 #include "universal.h"
 #include "spherical.h"
 #include "clamp.h"
+#include "contact.h"
+#include "conical.h"
+#include "polar.h"
 
 #include "angular.h"
+#include "euler.h"
 #include "linear.h"
 
 static char *decapitalize (char *s)
@@ -73,6 +77,18 @@ static int foundry_tostring(lua_State *L)
     return 1;
 }
 
+static int dummy_index(lua_State *L)
+{
+    lua_pushnil (L);
+    
+    return 1;
+}
+
+static int dummy_newindex(lua_State *L)
+{
+    return 0;
+}
+
 static int node_tostring(lua_State *L)
 {
     id object;
@@ -316,8 +332,21 @@ static int constructpolyhedron(lua_State *L)
     lua_pushstring(L, "__gc");
     lua_pushcfunction(L, foundry_gc);
     lua_settable(L, -3);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, dummy_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, dummy_newindex);
+    lua_settable(L, -3);
     lua_setmetatable(L, -2);
 
+    lua_pushstring (L, "userdata");
+    lua_gettable (L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (L, object);
+    lua_pushvalue (L, -3);
+    lua_settable (L, -3);
+    lua_pop(L, 1);
+
     free(vertices);
     free(indices);
 
@@ -500,8 +529,21 @@ int constructheightfield(lua_State *L)
     lua_pushstring(L, "__gc");
     lua_pushcfunction(L, foundry_gc);
     lua_settable(L, -3);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, dummy_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, dummy_newindex);
+    lua_settable(L, -3);
     lua_setmetatable(L, -2);
 
+    lua_pushstring (L, "userdata");
+    lua_gettable (L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (L, object);
+    lua_pushvalue (L, -3);
+    lua_settable (L, -3);
+    lua_pop(L, 1);
+
     return 1;
 }
 
@@ -687,8 +729,21 @@ int constructconvex(lua_State *L)
     lua_pushstring(L, "__gc");
     lua_pushcfunction(L, foundry_gc);
     lua_settable(L, -3);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, dummy_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, dummy_newindex);
+    lua_settable(L, -3);
     lua_setmetatable(L, -2);
 
+    lua_pushstring (L, "userdata");
+    lua_gettable (L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (L, object);
+    lua_pushvalue (L, -3);
+    lua_settable (L, -3);
+    lua_pop(L, 1);
+
     return 1;
 }
 
@@ -698,17 +753,18 @@ int luaopen_dynamics (lua_State *L)
     
     Class bodies[] = {
 	[System class], [Point class], [Ball class], [Box class],
-	[Environment class], [Capsule class],
-	[Ray class], [Plane class], [Composite class]
+	[Environment class], [Capsule class], [Ray class],
+	[Plane class], [Composite class]
     };	
     
     Class joints[] = {
 	[Slider class], [Universal class], [Hinge class], [Spherical class],
-	[Clamp class]
+	[Clamp class], [Contact class], [Conical class],
+	[Polar class]
     };	
     
     Class motors[] = {
-	[Angular class], [Linear class]
+	[Angular class], [Linear class], [Euler class]
     };	
 
     lua_newtable (L);
