@@ -96,35 +96,44 @@ static void calculate_view_frustum()
 	pi[i][3] /= m;
     }
 }
+ 
+static void switch_to_context(Elevation *new)
+{
+    context = new;
+}
 
 static void look_up_sample(int j, int i, float *h, float *e)
 {
     int a, b, c, d;
     unsigned int k, l, m;
-
+    
     a = context->size[0];
     b = context->size[1];
     c = context->size[2];
 /*     d = (1 << context->size[2]) + 1; */
     d = 1 << context->size[2];
 
-    /* Split the global index into patch/sample
-       indices and do the look-up. */
+    if (i >= 0 && i <= a * d && j >= 0 && j <= b * d) {
+	/* Split the global index into patch/sample
+	   indices and do the look-up. */
 	
-    k = ((i / d) * b) + (j / d);
-    
-    *h = 0.0;
-    *e = 0.0;
-    
-    if (i >= 0 && i < a * d &&
-	j >= 0 && j < b * d &&
-	context->samples[k]) {
+	k = ((i / d) * b) + (j / d);
 
-	l = context->orders[k];
-	m = ((i % d) << (l + l - c)) + (((i % d) + (j % d)) >> (c - l));
+	if (k >= a * b || !context->samples[k]) {
+	    d += 1;
+	    k = ((i / d) * b) + (j / d);
+	}
+    
+	*h = 0.0;
+	*e = 0.0;
+    
+	if (context->samples[k]) {
+	    l = context->orders[k];
+	    m = ((i % d) << (l + l - c)) + (((i % d) + (j % d)) >> (c - l));
 
-	*h = (float)(context->samples[k][m]);
-	*e = 0.5 * (float)(context->bounds[k][m]);
+	    *h = (float)(context->samples[k][m]);
+	    *e = (float)(context->bounds[k][m]);
+	}
     }
 }
 
@@ -132,6 +141,7 @@ static dReal heightfield_data_callback (void *data, int x, int z)
 {
     float h, e;
 
+    switch_to_context ((Elevation *)data);
     look_up_sample (x, z, &h, &e);
 
 /*     printf ("%d, %d => %f\n", x, z, h); */
@@ -471,6 +481,14 @@ static void initialize_triangle(Triangle *c, Diamond *d, Triangle *p,
     c->index = i;
     c->flags = 0;
     c->cullbits = OUT;
+
+    /* if (p && d->error > p->diamond->error + 0.25) { */
+    /* 	printf ("??? %p: %f, %f, (%f,%f)\n", */
+    /* 		d, d->error, p->diamond->error, */
+    /* 		d->center[0], d->center[1]); */
+    /* } else { */
+    /* 	puts("!!!"); */
+    /* } */
 }
 
 static void classify_triangle(Triangle *n, int b)
@@ -482,7 +500,7 @@ static void classify_triangle(Triangle *n, int b)
     if(b != OUT && b != ALL_IN) {
         p = planes;
         r0 = n->diamond->error;
-
+	
         v[0] = n->diamond->vertices[0];
         v[1] = n->diamond->vertices[1];
         v[2] = n->parent->diamond->center;
@@ -841,7 +859,7 @@ static void draw_subtree(Triangle *n)
 		glColor3f(1, 1, 1);
 	    }
 #endif    
-	    
+
 	    glVertex3fv(d->vertices[!i]);
 	    glVertex3fv(d->vertices[i]);
 	    glVertex3fv(e->center);
@@ -973,11 +991,6 @@ static void draw_earth_geometry()
     }
 }       
 
-static void switch_to_context(Elevation *new)
-{
-    context = new;
-}
-
 static void *allocate_mesh()
 {
     Triangle *(*T)[4];
@@ -1290,8 +1303,9 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
     self->resolution[2] = scale[2];
 
     self->data = dGeomHeightfieldDataCreate();
+
     dGeomHeightfieldDataBuildCallback (self->data,
-				       NULL, heightfield_data_callback,
+				       self, heightfield_data_callback,
 				       (1 << order) * width * scale[0],
 				       (1 << order) * height * scale[1],
 				       (1 << order) * width + 1,
@@ -1306,7 +1320,7 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 -(Elevation *) init
 {
     char *list[] = {
-	"detail", "orientation", "position", "target"
+	"detail", "target", "corporeal", "state"
     };
 
     [super init];
@@ -1332,6 +1346,8 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
     self->chunks[0] = 0;
     self->chunks[1] = 0;
 
+    self->corporeal = 1;
+    
     /* Create the collision detection geom. */
 
     self->geom = dCreateHeightfield(NULL, self->data, 1);
@@ -1366,6 +1382,8 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 {
     int i;
 
+    dGeomHeightfieldDataDestroy(self->data);
+
     glDeleteTextures (self->size[0] * self->size[1], self->imagery);
     glDeleteTextures (1, &self->detail);
     
@@ -1384,16 +1402,18 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 	switch_to_context(self);
 	free_mesh();
     }
-    
+        
     [super free];
 }
 
 -(void) toggle
 {
-    if (!linked) {
-	dSpaceAdd (_SPACE, self->geom);
-    } else {
-	dSpaceRemove (_SPACE, self->geom);
+    if (self->corporeal) {
+	if (!self->linked) {
+	    dSpaceAdd (_SPACE, self->geom);
+	} else {
+	    dSpaceRemove (_SPACE, self->geom);
+	}
     }
 
     [super toggle];
@@ -1434,13 +1454,15 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
     glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
     glPixelStorei(GL_UNPACK_ROW_LENGTH, length);
 
-    gluBuild2DMipmaps (GL_TEXTURE_2D,
-		       GL_RGB,
-		       length, length,
-		       GL_RGB,
-		       GL_UNSIGNED_BYTE,
-		       pixels);
-
+    if (_WINDOW) {
+	gluBuild2DMipmaps (GL_TEXTURE_2D,
+			   GL_RGB,
+			   length, length,
+			   GL_RGB,
+			   GL_UNSIGNED_BYTE,
+			   pixels);
+    }
+    
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
 		    GL_LINEAR_MIPMAP_LINEAR);
@@ -1476,12 +1498,14 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
     glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
     glPixelStorei(GL_UNPACK_ROW_LENGTH, length);
 
-    gluBuild2DMipmaps (GL_TEXTURE_2D,
-		       GL_LUMINANCE,
-		       length, length,
-		       GL_LUMINANCE,
-		       GL_UNSIGNED_BYTE,
-		       pixels);
+    if (_WINDOW) {
+	gluBuild2DMipmaps (GL_TEXTURE_2D,
+			   GL_LUMINANCE,
+			   length, length,
+			   GL_LUMINANCE,
+			   GL_UNSIGNED_BYTE,
+			   pixels);
+    }
 
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
@@ -1525,6 +1549,8 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 
     if (!xstrcmp(k, "target")) {
 	lua_pushnumber (_L, self->target);
+    } else if (!xstrcmp(k, "corporeal")) {
+	lua_pushboolean (_L, self->corporeal);
     } else if (!xstrcmp(k, "state")) {
 	lua_newtable (_L);
 
@@ -1704,6 +1730,14 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 	}	
     } else if (!xstrcmp(k, "target")) {
 	self->target = lua_tonumber (_L, -1);
+    } else if (!xstrcmp(k, "corporeal")) {
+	self->corporeal = lua_toboolean (_L, -1);
+
+	if (self->geom &&
+	    dGeomGetSpace (self->geom) &&
+	    !self->corporeal) {
+	    dSpaceRemove (_SPACE, self->geom);
+	}
     } else if (!xstrcmp(k, "position")) {
 	[super set];
 
@@ -1761,7 +1795,7 @@ static void traverse_quadtree(Triangle *n, Triangle *m, int l)
 	optimize_earth_geometry();
 	draw_earth_geometry();
 
-	/* glPolygonMode (GL_FRONT_AND_BACK, GL_FILL); */
+	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
 
 	glMatrixMode (GL_MODELVIEW);
 	glPopMatrix();
