@@ -46,7 +46,7 @@
     int triangles, diamonds, culled, visible;
     int minimum, maximum;
 
-    int target;
+    int target, corporeal;
 }
 
 -(Elevation *) initWith: (int)width
