@@ -157,6 +157,18 @@
     [super input];
 }
 
+-(void) free
+{
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->buttonpress);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->buttonrelease);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->keypress);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->keyrelease);
+    luaL_unref (_L, LUA_REGISTRYINDEX, motion);
+    luaL_unref (_L, LUA_REGISTRYINDEX, scroll);
+
+    [super free];
+}
+
 -(void) get
 {
     const char *k;
@@ -200,10 +212,10 @@
 	self->keyrelease = luaL_ref (_L, LUA_REGISTRYINDEX);
     } else if (!xstrcmp(k, "motion")) {
 	luaL_unref (_L, LUA_REGISTRYINDEX, motion);
-	motion = luaL_ref (_L, LUA_REGISTRYINDEX);
+	self->motion = luaL_ref (_L, LUA_REGISTRYINDEX);
     } else if (!xstrcmp(k, "scroll")) {
 	luaL_unref (_L, LUA_REGISTRYINDEX, scroll);
-	scroll = luaL_ref (_L, LUA_REGISTRYINDEX);
+	self->scroll = luaL_ref (_L, LUA_REGISTRYINDEX);
     } else {
 	[super set];
     }
