@@ -17,9 +17,9 @@
 #ifndef _FLUID_H_
 #define _FLUID_H_
 
-#include "node.h"
+#include "transform.h"
 
-@interface Fluid: Node {
+@interface Fluid: Transform {
     double *particles, *subparticles, tick, support, mobility;
     double scale[2], state[2], coefficients[2];
     double viscosity[2], stiffness[2], density, diffusion;
