@@ -75,6 +75,8 @@
     k = lua_tostring (_L, -1);
 
     if (!xstrcmp(k, "anchor")) {
+	dJointGetHingeAnchor ([self joint], self->anchor);
+	
         lua_newtable (_L);
         
         for(i = 0; i < 3; i += 1) {
@@ -82,6 +84,8 @@
             lua_rawseti (_L, -2, i + 1);
         }
     } else if (!xstrcmp(k, "axis")) {
+	dJointGetHingeAxis ([self joint], self->axis);
+	
         lua_newtable (_L);
         
         for(i = 0; i < 3; i += 1) {
@@ -221,10 +225,12 @@
 				 self->stops[0]);
 	    dJointSetHingeParam ([self joint], dParamHiStop,
 				 self->stops[1]);
+
 	    dJointSetHingeParam ([self joint], dParamStopCFM,
 				 self->hardness[0]);
 	    dJointSetHingeParam ([self joint], dParamStopERP,
 				 self->hardness[1]);
+
 	    dJointSetHingeParam ([self joint], dParamBounce, self->bounce);
 	}
     } else if (!xstrcmp(k, "tolerance")) {
@@ -261,6 +267,7 @@
 	glEnable(GL_BLEND);
 	glDepthMask (GL_FALSE);
 
+	glMatrixMode (GL_MODELVIEW);
 	glPushMatrix();
 	glTranslatef(p[0], p[1], p[2]);
 	
@@ -278,7 +285,11 @@
 
 	glLineWidth (1);
 
-	glColor3f(0, 0, 1);
+	if (self->tag > 0) {
+	    glColor3f(1, 1, 0);
+	} else {
+	    glColor3f(0, 0, 1);
+	}
 
 	glBegin (GL_LINES);
 	glVertex3f (0, 0, 0);
@@ -307,7 +318,7 @@
 
 	    /* Draw the other arm and anchor. */
 	
-	    glColor3f(0, 1, 0);
+	    glColor3f(self->tag, 1, 0);
 	    glLineWidth (3);
 
 	    glBegin (GL_LINES);
