@@ -17,6 +17,12 @@
 #include <assert.h>
 #include "item.h"
 
+/* This is a node that keeps track of all allocated
+   Nodes by adopting them (and their descendants)
+   when they become orphans. */
+
+static Item *subroot = nil;
+
 @implementation Item
 
 -forward: (SEL) sel : args
@@ -24,6 +30,11 @@
     return nil;
 }
 
++(void) initialize
+{
+    subroot = [[Item alloc] init];
+}
+
 -(Item *) init
 {
     [super init];
@@ -35,6 +46,10 @@
     self->down = NULL;
     self->left = NULL;
     self->right = NULL;
+
+    if (subroot) {
+	[subroot adopt: self named: [self name]];
+    }
     
     return self;
 }
@@ -43,6 +58,14 @@
 {
     free (self->properties);
 
+    /* Remove dead children from the subroot.
+       Note that dead notes may belong to other
+       parents when whole subtrees are collected. */
+    
+    if (self->up == subroot) {
+    	[subroot renounce: self];
+    }
+
     [super free];
 }
 
@@ -74,8 +97,12 @@
 -(id) root
 {
     id root;
+
+    /* Return the proper graph root, not the subroot. */
     
-    for (root = self ; [root parent] ; root = [root parent]);
+    for (root = self;
+	 [root parent] && [root parent] != subroot;
+	 root = [root parent]);
 
     return (id)root;
 }
@@ -92,16 +119,16 @@
 
 -(void) add: (int) n Properties: (char*[]) list
 {
-    /* int i; */
+    int i;
 
-    /* self->properties = realloc (self->properties, */
-    /* 				(self->bloat + n) * sizeof (char *)); */
+    self->properties = realloc (self->properties,
+    				(self->bloat + n) * sizeof (char *));
 
-    /* for (i = 0 ; i < n ; i += 1) { */
-    /* 	self->properties[self->bloat + i] = list[i]; */
-    /* } */
+    for (i = 0 ; i < n ; i += 1) {
+    	self->properties[self->bloat + i] = list[i];
+    }
 
-    /* self->bloat += n; */
+    self->bloat += n;
 }
 
 -(void) adoptedBy: (Item *)parent
@@ -111,6 +138,14 @@
 -(void) adopt: (Item *)child named: (char *)name
 {
     Item *sibling;
+
+    /* Before the child can get adopted by a proper
+       parent the subroot needs to let go of it. */
+    
+    assert (subroot);
+    if (self != subroot) {
+    	[subroot renounce: child];
+    }
     
     child->key = name;
     child->up = self;
@@ -159,6 +194,14 @@
     child->left = NULL;
     child->right = NULL;
     child->up = NULL;
+
+    /* Put any foster nodes under the care of the
+       subroot. */
+    
+    assert (subroot);
+    if (self != subroot) {
+    	[subroot adopt: child named: [self name]];
+    }
 }
 
 @end
