@@ -99,6 +99,8 @@ static void callhooks (lua_State *L, void *self, void *a, void *b, int reference
 -(void) free
 {
     dJointDestroy (self->joint);
+
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->attach);
     
     [super free];
 }
@@ -132,6 +134,8 @@ static void callhooks (lua_State *L, void *self, void *a, void *b, int reference
     
 	if (q) {
 	    b = [q body];
+	} else if (self->body) {
+	    b = [self->body body];
 	} else {
 	    b = NULL;
 	}
@@ -238,6 +242,12 @@ static void callhooks (lua_State *L, void *self, void *a, void *b, int reference
 	lua_rawseti (_L, -2, 2);	
     } else if (!xstrcmp(k, "inverted")) {
 	lua_pushboolean (_L, self->inverted);
+    } else if (!xstrcmp(k, "reference")) {	
+	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__reference");
+	lua_replace (_L, 2);
+        lua_rawget (_L, 1);	
     } else if (!xstrcmp(k, "attach")) {
         lua_rawgeti (_L, LUA_REGISTRYINDEX, self->attach);
     } else {
@@ -255,6 +265,20 @@ static void callhooks (lua_State *L, void *self, void *a, void *b, int reference
     if (!xstrcmp(k, "inverted")) {
 	self->inverted = lua_toboolean (_L, 3);
 
+	[self update];
+    } else if (!xstrcmp(k, "reference")) {
+	if (lua_isuserdata(_L, 3)) {
+	    self->body = *(id *)lua_touserdata(_L, 3);
+	} else {
+	    self->body = nil;
+	}
+	
+	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__reference");
+	lua_replace (_L, 2);
+        lua_rawset (_L, 1);	
+
 	[self update];
     } else if (!xstrcmp(k, "attach")) {
         luaL_unref (_L, LUA_REGISTRYINDEX, self->attach);
