@@ -213,7 +213,7 @@
     
 	glUseProgramObjectARB(0);
 
-	for (root = self ; [root parent] ; root = [root parent]);
+	root = [self root];
 	
 	[[self parent] traversePass: 0];
     
