@@ -60,6 +60,23 @@
     return self;
 }
 
+-(void) update
+{
+  int i;
+
+    [super update];
+
+    /* Reset the axis. */
+	
+    for (i = 0 ; i < self->degrees ; i += 1) {
+      dJointSetLMotorAxis ([self joint], i,
+			   dJointGetBody ([self joint], 0) ? 1 : 0,
+			   self->axes[i][0],
+			   self->axes[i][1],
+			   self->axes[i][2]);
+    }
+}
+
 -(void) get
 {
     const char *k;
@@ -274,16 +291,6 @@
 				      self->tolerance[j]);
             }
         }
-    } else if (!xstrcmp(k, "bodies")) {
-	[super set];
-
-	for (i = 0 ; i < self->degrees ; i += 1) {
-	    dJointSetLMotorAxis ([self joint], i,
-				 dJointGetBody ([self joint], 0) ? 1 : 0,
-				 self->axes[i][0],
-				 self->axes[i][1],
-				 self->axes[i][2]);
-	}
     } else {
 	[super set];
     }
