@@ -47,13 +47,13 @@ static jmp_buf env;
 
 static ALCdevice *device;
 
-static int iterate = 1, dying, frames, width, height, decorate;
+static int iterate = 1, frames, width, height, decorate;
 static int focus = LUA_REFNIL, defocus = LUA_REFNIL, delete = LUA_REFNIL;
 static float canvas[3];
-static char *title;
+static char *title, *phase;
 static const char *devicename;
 
-static double now, then, intervals[19], userinterval, interval = -1;
+static double now, then, intervals[21], userinterval, interval = -1;
 static struct timespec zero;
 
 static double timescale = 1, stepsize = 0.01, ceiling = 1.0 / 0.0;
@@ -64,13 +64,8 @@ static void callback (void *data, dGeomID a, dGeomID b)
 {
     dContactGeom geoms[16];
     dContact contacts[16];
-    dReal mu, epsilon, sigma, tau;
+    dReal mu = 0, epsilon = 0, sigma = 0, tau = 1;
     int i, j, n;
-
-    mu = 0.0;
-    epsilon = 0.0;
-    sigma = 0.0;
-    tau = 1.0;
     
     if (dGeomIsSpace (a) || dGeomIsSpace (b)) {
 	/* Recurse into subspaces as needed. */
@@ -232,7 +227,7 @@ static void callback (void *data, dGeomID a, dGeomID b)
 			contacts[i].surface.mode |= dContactSoftERP;
 			contacts[i].surface.soft_erp = tau;
 		    }
-		    
+
 		    joint = dJointCreateContact (_WORLD, _GROUP, &contacts[i]);
 		    dJointAttach (joint, dGeomGetBody (a), dGeomGetBody (b));
 		}
@@ -255,17 +250,36 @@ static int panic(lua_State *L)
     /* Don't let Lua terminate the process yet.
        There's cleaning up to do. */
     
-    longjmp (env, 1);
-
+    /* longjmp (env, 1); */
+    
     return 0;
 }
 
 static void handler(int signum)
 {
+    struct sigaction action;
+    
     printf("Hmmm, I just received a rather lethal signal "
 	   "(number %d if you must know).\n",
 	   signum);
 
+    /* Disable the handler for further occurences of this signal. */
+    
+    action.sa_handler = SIG_DFL;
+    action.sa_flags = 0;
+    sigemptyset (&action.sa_mask);
+
+    sigaction (SIGHUP, &action, NULL);
+    sigaction (SIGINT, &action, NULL);
+    sigaction (SIGQUIT, &action, NULL);
+    sigaction (SIGILL, &action, NULL);
+    sigaction (SIGABRT, &action, NULL);
+    sigaction (SIGKILL, &action, NULL);
+    sigaction (SIGSEGV, &action, NULL);
+    sigaction (SIGPIPE, &action, NULL);
+    sigaction (SIGALRM, &action, NULL);
+    sigaction (SIGTERM, &action, NULL);
+    
     /* Don't terminate the process yet.
        There's cleaning up to do. */
     
@@ -407,7 +421,7 @@ static int properties(lua_State *L)
 
 static int root_tostring(lua_State *L)
 {
-    lua_pushstring(L, "root");
+    lua_pushstring(L, "Root");
    
     return 1;
 }
@@ -432,77 +446,6 @@ static int root_newindex(lua_State *L)
     return 0;
 }
 
-static int iteration_index (lua_State *L)
-{
-    const char *k;
-
-    k = lua_tostring(L, 2);
-
-    if (!xstrcmp(k, "input")) {
-	lua_pushnumber (L, intervals[0]);
-    } else if (!xstrcmp(k, "begin")) {
-	lua_pushnumber (L, intervals[1]);
-    } else if (!xstrcmp(k, "transform")) {
-	lua_pushnumber (L, intervals[2]);
-    } else if (!xstrcmp(k, "collide")) {
-	lua_pushnumber (L, intervals[3]);
-    } else if (!xstrcmp(k, "step")) {
-	lua_pushnumber (L, intervals[4]);
-    } else if (!xstrcmp(k, "integrate")) {
-	lua_pushnumber (L, intervals[5]);
-    } else if (!xstrcmp(k, "prepare")) {
-	lua_pushnumber (L, intervals[6]);
-    } else if (!xstrcmp(k, "traverse")) {
-	lua_newtable (L);
-
-	lua_pushnumber (L, intervals[7]);
-	lua_rawseti (L, -2, 1);
-	lua_pushnumber (L, intervals[8]);
-	lua_rawseti (L, -2, 2);
-    } else if (!xstrcmp(k, "finish")) {
-	lua_pushnumber (L, intervals[9]);
-    } else if (!xstrcmp(k, "total")) {
-	lua_pushnumber (L, intervals[10]);
-    } else if (!xstrcmp(k, "user")) {
-	int i;
-	
-	lua_newtable (L);
-
-	for (i = 0 ; i < 8 ; i += 1) {
-	    lua_pushnumber (L, intervals[11 + i]);
-	    lua_rawseti (L, -2, i + 1);
-	}
-    } else {
-	lua_rawget (L, 1);
-    }
-
-    return 1;
-}
-
-static int iteration_newindex (lua_State *L)
-{
-    const char *k;
-
-    k = lua_tostring(L, 2);
-
-    if (!xstrcmp(k, "input")) {
-    } else if (!xstrcmp(k, "begin")) {
-    } else if (!xstrcmp(k, "transform")) {
-    } else if (!xstrcmp(k, "collide")) {
-    } else if (!xstrcmp(k, "step")) {
-    } else if (!xstrcmp(k, "integrate")) {
-    } else if (!xstrcmp(k, "prepare")) {
-    } else if (!xstrcmp(k, "traverse")) {
-    } else if (!xstrcmp(k, "finish")) {
-    } else if (!xstrcmp(k, "total")) {
-    } else if (!xstrcmp(k, "user")) {
-    } else {
-	lua_rawset (L, 1);
-    }
-
-    return 0;
-}
-
 static int accoustics_index (lua_State *L)
 {
     const char *k;
@@ -536,9 +479,57 @@ static int accoustics_newindex (lua_State *L)
     return 0;
 }
 
+static void census (lua_State *L, id node)
+{
+    id child;
+    
+    /* Skip the subroot. */
+
+    if (node != [root parent]) {
+	/* Get the ledger entry for the node's class. */
+	
+	lua_pushstring (_L, [node name]);
+	lua_gettable (_L, -2);
+
+	/* If there isn't one create it. */
+	
+	if (lua_isnil (_L, -1)) {
+	    lua_pop (_L, 1);
+
+	    lua_pushstring (_L, [node name]);
+	    lua_newtable (_L);
+	    lua_settable (_L, -3);
+	    
+	    lua_pushstring (_L, [node name]);
+	    lua_gettable (_L, -2);
+	}
+
+	/* Get the node's userdata. */
+	
+	lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
+	lua_pushlightuserdata (_L, node);
+	lua_gettable (_L, -2);
+	lua_replace (_L, -2);
+
+	if (lua_isnil (_L, -1)) {
+	    printf ("WARNING: Could not get userdata for node %p of class %s\n", node, [node name]);
+	}
+
+	/* Enter the node and pop the entry. */
+	
+	lua_rawseti (_L, -2, luaX_objlen (_L, -2) + 1);
+	lua_pop (_L, 1);
+    }
+    
+    for(child = [node children] ; child ; child = [child sister]) {
+	census (L, child);
+    }
+}
+
 static int configuration_index (lua_State *L)
 {
     const char *k;
+    int i, j;
 
     k = lua_tostring(L, 2);
 
@@ -565,7 +556,6 @@ static int configuration_index (lua_State *L)
 	lua_pushstring (L, (const char *)alGetString(AL_EXTENSIONS));
 	lua_rawseti (L, -2, 4);
     } else if (!xstrcmp(k, "toolkit")) {
-	int i, j;
 	double v;
 	
         gdk_gl_query_version(&i, &j);
@@ -579,6 +569,43 @@ static int configuration_index (lua_State *L)
 	lua_rawseti (L, -2, 1);
 	lua_pushinteger (L, gdk_screen_height());
 	lua_rawseti (L, -2, 2);	
+    } else if (!xstrcmp(k, "iteration")) {
+	lua_newtable (L);
+
+	for (i = 0;
+	     i < sizeof (intervals) / sizeof (intervals[0]);
+	     i += 1) {
+	    lua_pushnumber (L, intervals[i]);
+	    lua_rawseti (L, -2, i + 1);
+	}
+    } else if (!xstrcmp(k, "time")) {
+	struct timespec now;
+
+	lua_newtable (L);
+
+	/* Simulation time. */
+	
+	lua_pushnumber (L, then);
+	lua_rawseti (L, -2, 1);
+
+	/* Wall clock time. */
+	
+	clock_gettime (CLOCK_REALTIME, &now);
+	lua_pushnumber (L,
+			now.tv_sec - zero.tv_sec +
+			(now.tv_nsec - zero.tv_nsec) / 1e9);
+	lua_rawseti (L, -2, 2);
+	
+	/* CPU time for process. */
+	
+	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &now);
+	lua_pushnumber (L, now.tv_sec + now.tv_nsec / 1e9);
+	lua_rawseti (L, -2, 3);
+    } else if (!xstrcmp(k, "nodes")) {
+	lua_newtable (L);
+	census (L, [root parent]);	
+    } else if (!xstrcmp(k, "phase")) {
+	lua_pushstring (L, phase);
     } else {
 	lua_rawget (L, 1);
     }
@@ -596,50 +623,11 @@ static int configuration_newindex (lua_State *L)
     } else if (!xstrcmp(k, "accoustics")) {
     } else if (!xstrcmp(k, "toolkit")) {
     } else if (!xstrcmp(k, "screen")) {
-    } else {
-	lua_rawset (L, 1);
-    }
-
-    return 0;
-}
-
-static int time_index (lua_State *L)
-{
-    const char *k;
-
-    k = lua_tostring(L, 2);
-
-    if (!xstrcmp(k, "real")) {
-	struct timespec now;
-
-	clock_gettime (CLOCK_REALTIME, &now);
-	lua_pushnumber (L,
-			now.tv_sec - zero.tv_sec +
-			(now.tv_nsec - zero.tv_nsec) / 1e9);
-    } else if (!xstrcmp(k, "process")) {
-	struct timespec now;
-
-	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &now);
-	lua_pushnumber (L, now.tv_sec + now.tv_nsec / 1e9);
-    } else if (!xstrcmp(k, "physics")) {
-	lua_pushnumber (L, then);
-    } else {
-	lua_rawget (L, 1);
-    }
-
-    return 1;
-}
-
-static int time_newindex (lua_State *L)
-{
-    const char *k;
-    
-    k = lua_tostring(L, 2);
-
-    if (!xstrcmp(k, "real")) {
-    } else if (!xstrcmp(k, "process")) {
-    } else if (!xstrcmp(k, "physics")) {
-    } else {
+    } else if (!xstrcmp(k, "iteration")) {
+    } else if (!xstrcmp(k, "time")) {
+    } else if (!xstrcmp(k, "nodes")) {
+    } else if (!xstrcmp(k, "phase")) {
+   } else {
 	lua_rawset (L, 1);
     }
 
@@ -810,6 +798,8 @@ static int graphics_index (lua_State *L)
 	lua_pushboolean (L, decorate);
     } else if (!xstrcmp(k, "cursor")) {
 	lua_pushboolean (L, cursor);
+    } else if (!xstrcmp(k, "grabinput")) {
+	lua_pushboolean (L, gdk_pointer_is_grabbed ());
     } else if (!xstrcmp(k, "canvas")) {
 	lua_newtable(L);
         
@@ -915,17 +905,17 @@ static int graphics_newindex (lua_State *L)
 	    gdk_window_set_geometry_hints(_WINDOW, &geometry,
 	    				  GDK_HINT_MIN_SIZE |
 	    				  GDK_HINT_MAX_SIZE);
-	    
-	    if (width == gdk_screen_width() &&
-		height == gdk_screen_height()) {
-		gdk_window_fullscreen (_WINDOW);
-	    }
 
 	    if (!hide) {
 		gdk_window_clear (_WINDOW);
 		gdk_window_show (_WINDOW);
 		gdk_window_raise (_WINDOW);
 		/* gdk_window_focus (_WINDOW, GDK_CURRENT_TIME); */
+
+		if (width == gdk_screen_width() &&
+		    height == gdk_screen_height()) {
+		    gdk_window_fullscreen (_WINDOW);
+		}
 	    }
 	}
 
@@ -978,6 +968,16 @@ static int graphics_newindex (lua_State *L)
 	decorate = lua_toboolean (L, 3);
 
 	gdk_window_set_override_redirect (_WINDOW, !decorate);
+    } else if (!xstrcmp(k, "grabinput")) {
+	if (lua_toboolean (_L, 3)) {
+	    gdk_pointer_grab (_WINDOW, TRUE,
+			      GDK_BUTTON_PRESS_MASK |
+			      GDK_BUTTON_RELEASE_MASK |
+			      GDK_POINTER_MOTION_MASK,
+			      NULL, NULL, GDK_CURRENT_TIME);
+	} else {
+	    gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	}
     } else if (!xstrcmp(k, "cursor")) {
 	GdkCursor *new;
 
@@ -1157,7 +1157,7 @@ int main(int argc, char **argv)
 
     lua_getglobal (_L, "package");
     lua_getfield (_L, -1, "path");
-    lua_pushstring (_L, ";"PKGDATADIR"/common/?.lua;"PKGDATADIR"/common/?.lc");
+    lua_pushstring (_L, ";"PKGDATADIR"/modules/?.lua;"PKGDATADIR"/modules/?.lc");
     lua_concat (_L, 2);
     lua_setfield (_L, -2, "path");
     lua_pop (_L, 1);
@@ -1341,7 +1341,7 @@ int main(int argc, char **argv)
     *(id *)lua_newuserdata (_L, sizeof(id)) = root;
     
     lua_newtable (_L);
-    lua_pushstring (_L, "__string");
+    lua_pushstring (_L, "__tostring");
     lua_pushcfunction (_L, (lua_CFunction)root_tostring);
     lua_settable (_L, -3);
     lua_pushstring (_L, "__index");
@@ -1352,6 +1352,13 @@ int main(int argc, char **argv)
     lua_settable (_L, -3);
     lua_setmetatable (_L, -2);
     
+    lua_pushstring (_L, "userdata");
+    lua_gettable (_L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (_L, root);
+    lua_pushvalue (_L, -3);
+    lua_settable (_L, -3);
+    lua_pop(_L, 1);
+
     [root toggle];
 
     lua_setglobal (_L, "graph");
@@ -1410,7 +1417,7 @@ int main(int argc, char **argv)
 	    "canvas", "close", "colorbuffer", "defocus", "extensions",
 	    "focus", "frames", "hide", "version", "window",
 	    "orthographic", "perspective", "renderer", "title", "cursor",
-	    "vendor"
+	    "vendor", "grabinput"
 	};
 
 	lua_newtable (_L);
@@ -1432,65 +1439,6 @@ int main(int argc, char **argv)
     
     lua_setglobal (_L, "graphics");
 
-    /* Create the time table. */
-
-    lua_newtable (_L);    
-
-    {
-	char *list[] = {
-	    "physics", "real", "process"
-	};
-
-	lua_newtable (_L);
-
-	lua_pushinteger (_L, sizeof (list) / sizeof (char *));
-	lua_setfield (_L, -2, "__bloat");
-
-	lua_pushlightuserdata (_L, list);
-	lua_setfield (_L, -2, "__properties");
-    
-	lua_pushcfunction (_L, time_index);
-	lua_setfield (_L, -2, "__index");
-
-	lua_pushcfunction (_L, time_newindex);
-	lua_setfield (_L, -2, "__newindex");
-	
-	lua_setmetatable (_L, -2);
-    }
-
-    lua_setglobal (_L, "time");
-
-    /* Create the profiling table. */
-
-    lua_newtable (_L);    
-    lua_newtable (_L);    
-
-    {
-	char *list[] = {
-	    "begin", "collide", "finish", "input", "integrate", "prepare",
-	    "step", "total", "transform", "traverse", "user"
-	};
-	
-	lua_newtable (_L);
-
-	lua_pushinteger (_L, sizeof (list) / sizeof (char *));
-	lua_setfield (_L, -2, "__bloat");
-
-	lua_pushlightuserdata (_L, list);
-	lua_setfield (_L, -2, "__properties");
-    
-	lua_pushcfunction (_L, iteration_index);
-	lua_setfield (_L, -2, "__index");
-
-	lua_pushcfunction (_L, iteration_newindex);
-	lua_setfield (_L, -2, "__newindex");
-	
-	lua_setmetatable (_L, -2);
-    }
-
-    lua_setfield (_L, -2, "iteration");
-    lua_setglobal (_L, "profiling");
-
     /* Create the configuration table. */
 
     lua_newtable (_L);    
@@ -1498,6 +1446,7 @@ int main(int argc, char **argv)
     {
 	char *list[] = {
 	    "version", "graphics", "accoustics", "toolkit", "screen"
+	    "time", "iteration", "nodes", "phase"
 	};
     
 	lua_newtable (_L);
@@ -1668,17 +1617,24 @@ int main(int argc, char **argv)
     clock_getres(CLOCK_PROCESS_CPUTIME_ID, &time);
     printf ("The resolution of the process clock is %ld ns\n", time.tv_nsec);
 
-    /* Set up signal handlers. */
+    /* Set up signal handlers for all
+       terminal signals so that we at
+       least attempt to clean up first. */
 
     action.sa_handler = handler;
     action.sa_flags = 0;
     sigemptyset (&action.sa_mask);
-
-    sigaction (SIGSEGV, &action, NULL);
+    
+    sigaction (SIGHUP, &action, NULL);
     sigaction (SIGINT, &action, NULL);
-    sigaction (SIGTERM, &action, NULL);
-    sigaction (SIGABRT, &action, NULL);
     sigaction (SIGQUIT, &action, NULL);
+    sigaction (SIGILL, &action, NULL);
+    sigaction (SIGABRT, &action, NULL);
+    sigaction (SIGKILL, &action, NULL);
+    sigaction (SIGSEGV, &action, NULL);
+    sigaction (SIGPIPE, &action, NULL);
+    sigaction (SIGALRM, &action, NULL);
+    sigaction (SIGTERM, &action, NULL);
 
     /* Save the stack at this point so that we can return
        here to clean up in case something goes wrong. */
@@ -1690,10 +1646,9 @@ int main(int argc, char **argv)
 	
 	iterate = 0;
     } else if (setjmp(env)) {
-	printf ("\nNo point in prolonging this dreary existence I suppose...\n");
+	printf ("\nAttempting to go down gracefully...\n");
 	
 	iterate = 0;
-	dying = 1;
     } else {
 	printf ("\nReading your scripts and hoping for the best.\n");
   
@@ -1704,18 +1659,52 @@ int main(int argc, char **argv)
 	/* Run the boot scripts. */
 
 	for (i = 0 ; i < scripts ; i += 1) {
+	    const char *prefixes[] = {"./", PKGDATADIR"/scripts/", ""};
+	    char *script = NULL;
+	    int j;
+	    
 	    if (names[i] == NULL && !feof (stdin)) {
 		printf ("Waiting for input on the command line...\n\n");
+	    } else {
+		/* Try to find the specified script in a bunch
+		   of predetermined directories. */
+		
+		for (j = 0;
+		     j < sizeof(prefixes) / sizeof (char *);
+		     j += 1) {
+		    script = realloc (script,
+				      strlen (prefixes[j]) +
+				      strlen (names[i]) + 1);
+
+		    strcpy (script, prefixes[j]);
+		    strcat (script, names[i]);
+
+		    if (!access (script, F_OK)) {
+			break;
+		    }
+		}
 	    }
 
-	    if(!luaL_loadfile (_L, names[i])) {
+	    if(!luaL_loadfile (_L, script)) {
 		luaX_call (_L, 0, 0);
 	    } else {
 		lua_error (_L);
 	    }
-	}
 
-	printf ("\nDone configuring, entering the main loop.\n");
+	    if (script) {
+		free (script);
+	    }
+	}
+    }
+
+    if (iterate) {
+	struct timespec now;
+
+	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &now);
+	
+	printf ("Spent %f CPU seconds initializing.\nNow entering the main loop.\n",
+		now.tv_sec + now.tv_nsec / 1e9);
+	
     }
 
     /* Enter the main loop. */
@@ -1725,6 +1714,8 @@ int main(int argc, char **argv)
 
 	userinterval = 0;
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[0]);
+
+	phase = "input";
 	
 	if (_WINDOW) {
 	    GdkEvent *event;
@@ -1790,6 +1781,7 @@ int main(int argc, char **argv)
 
 	userinterval = 0;
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);
+	phase = "begin";
 	[root begin];
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
 	intervals[12] = userinterval;
@@ -1806,7 +1798,7 @@ int main(int argc, char **argv)
 	}
 
 	/* Calculate the time to advance for this iteration. */
-	
+
 	if (delta > ceiling) {
 	    /* printf ("Skewing the simulation clock by %.3fs\n", */
 	    /* 	    ceiling - delta); */
@@ -1826,7 +1818,6 @@ int main(int argc, char **argv)
 	intervals[15] = 0;
 	
 	/* Simulate the system forward. */
-	
 	for (delta = now - then;
 	     delta >= stepsize;
 	     then += stepsize, delta -= stepsize) {
@@ -1836,7 +1827,11 @@ int main(int argc, char **argv)
 	    
 	    userinterval = 0;
 	    clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);	
+
+	    phase = "transform";
+
 	    [root transform];
+
 	    clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
 	    intervals[13] += userinterval;
 	    intervals[2] += (timers[2].tv_sec - timers[1].tv_sec +
@@ -1868,6 +1863,7 @@ int main(int argc, char **argv)
 
 	    userinterval = 0;
 	    clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);	
+	    phase = "step";
 	    [root stepBy: stepsize at: then];
 	    clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
 	    intervals[15] += userinterval;
@@ -1899,6 +1895,7 @@ int main(int argc, char **argv)
 
 	userinterval = 0;
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);
+	phase = "prepare";
 	[root prepare];
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
 	intervals[16] = userinterval;
@@ -1907,23 +1904,27 @@ int main(int argc, char **argv)
 	
 	userinterval = 0;
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);
+	phase = "pass";
 	[root traversePass: 1];
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
+	intervals[17] = userinterval;
 	intervals[7] = (timers[2].tv_sec - timers[1].tv_sec +
 			(timers[2].tv_nsec - timers[1].tv_nsec) / 1e9);
 	
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);
+	phase = "repass";
 	[root traversePass: 2];
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
-	intervals[17] = userinterval;
+	intervals[18] = userinterval;
 	intervals[8] = (timers[2].tv_sec - timers[1].tv_sec +
 			(timers[2].tv_nsec - timers[1].tv_nsec) / 1e9);
 	
 	userinterval = 0;
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);
+	phase = "finish";
 	[root finish];
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[2]);
-	intervals[18] = userinterval;
+	intervals[19] = userinterval;
 	intervals[9] = (timers[2].tv_sec - timers[1].tv_sec +
 			(timers[2].tv_nsec - timers[1].tv_nsec) / 1e9);
 
@@ -1936,19 +1937,17 @@ int main(int argc, char **argv)
 	clock_gettime (CLOCK_PROCESS_CPUTIME_ID, &timers[1]);
 	intervals[10] = (timers[1].tv_sec - timers[0].tv_sec +
 			(timers[1].tv_nsec - timers[0].tv_nsec) / 1e9);
+
+	for (intervals[20] = 0, i = 11;
+	     i < 20;
+	     intervals[20] += intervals[i], i += 1);
 	
 	frames += 1;
     }
 
-    /* Unlink the whole scene unless we
-       ended up here because of an error
-       in which case it might not be a
-       good idea to run any node-related
-       code. */
+    /* Unlink the whole scene. */
 
-    if (!dying) {
-	[root toggle];
-    }
+    [root toggle];
 
     /* Finish cleaning up. */
     
