@@ -66,6 +66,18 @@ static int generic_gc(lua_State *L)
     return 0;
 }
 
+static int dummy_index(lua_State *L)
+{
+    lua_pushnil (L);
+    
+    return 1;
+}
+
+static int dummy_newindex(lua_State *L)
+{
+    return 0;
+}
+
 static int static_len(lua_State *L)
 {
     id object;
@@ -144,7 +156,7 @@ static int static_tostring(lua_State *L)
     object = *(id *)lua_touserdata(L, 1);
 
     lua_pushstring(L, [object name]);
-    lua_pushstring(L, " geometry foundry");
+    lua_pushstring(L, " foundry");
     lua_concat(L, 2);
     
     return 1;
@@ -269,8 +281,21 @@ static int constructstatic(lua_State *L)
     lua_pushstring(L, "__gc");
     lua_pushcfunction(L, static_gc);
     lua_settable(L, -3);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, dummy_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, dummy_newindex);
+    lua_settable(L, -3);
     lua_setmetatable(L, -2);
 
+    lua_pushstring (L, "userdata");
+    lua_gettable (L, LUA_REGISTRYINDEX);
+    lua_pushlightuserdata (L, object);
+    lua_pushvalue (L, -3);
+    lua_settable (L, -3);
+    lua_pop(L, 1);
+
     free(vertices);
     free(indices);
 
