@@ -76,6 +76,18 @@ static int generic_gc(lua_State *L)
     return 0;
 }
 
+static int dummy_index(lua_State *L)
+{
+    lua_pushnil (L);
+    
+    return 1;
+}
+
+static int dummy_newindex(lua_State *L)
+{
+    return 0;
+}
+
 static int elevation_len(lua_State *L)
 {
     id M;
@@ -143,7 +155,13 @@ static int elevation_call (lua_State *L)
 
 static int elevation_tostring(lua_State *L)
 {
-    lua_pushstring(L, "Elevation geometry foundry");
+    id object;
+
+    object = *(id *)lua_touserdata(L, 1);
+
+    lua_pushstring(L, [object name]);
+    lua_pushstring(L, " foundry");
+    lua_concat(L, 2);
     
     return 1;
 }
@@ -216,12 +234,18 @@ static int constructelevation(lua_State *L)
     lua_pushstring(L, "__gc");
     lua_pushcfunction(L, elevation_gc);
     lua_settable(L, -3);
+    lua_pushstring(L, "__index");
+    lua_pushcfunction(L, dummy_index);
+    lua_settable(L, -3);
+    lua_pushstring(L, "__newindex");
+    lua_pushcfunction(L, dummy_newindex);
+    lua_settable(L, -3);
     lua_setmetatable(L, -2);
 
     lua_pushstring (L, "userdata");
     lua_gettable (L, LUA_REGISTRYINDEX);
     lua_pushlightuserdata (L, object);
-    lua_pushvalue (L, 2);
+    lua_pushvalue (L, -3);
     lua_settable (L, -3);
     lua_pop(L, 1);
 
