@@ -346,7 +346,7 @@ static int respond (void *cls,
 		 s;
 		 s = strstr(lua_tostring(_L, -1), "<?lua")) {
 		struct context chunk;
-		const char *e;
+		const char *e, *t;
 		int l;
 		
 		/* printf ("** %d\n", lua_gettop(_L)); */
@@ -357,7 +357,9 @@ static int respond (void *cls,
 
 		lua_pushfstring (_L, "=[%s: %d]", url, l);
 
-		e = strstr(s, "?>");
+		for (e = strstr(s, "?>"), t = strstr(s + 2, "<?");
+		     e && t && e > t;
+		     t = strstr(e + 2, "<?"), e = strstr(e + 2, "?>"));
 
 		if (e != NULL) {
 		    chunk.data = s + 5;
@@ -568,6 +570,31 @@ static int generic_gc(lua_State *L)
     return 0;
 }
 
+static int dummy_index(lua_State *L)
+{
+    lua_pushnil (L);
+    
+    return 1;
+}
+
+static int dummy_newindex(lua_State *L)
+{
+    return 0;
+}
+
+static int peer_tostring(lua_State *L)
+{
+    id object;
+
+    object = *(id *)lua_touserdata(L, 1);
+
+    lua_pushstring(L, [object name]);
+    lua_pushstring(L, " foundry");
+    lua_concat(L, 2);
+    
+    return 1;
+}
+
 static int peer_gc(lua_State *L)
 {
     id object;
@@ -830,12 +857,25 @@ static int constructpeer(lua_State *L)
 	lua_pushcfunction(L, peer_call);
 	lua_settable(L, -3);
 	lua_pushstring(L, "__tostring");
-	lua_pushcfunction(L, generic_tostring);
+	lua_pushcfunction(L, peer_tostring);
 	lua_settable(L, -3);
 	lua_pushstring(L, "__gc");
 	lua_pushcfunction(L, peer_gc);
 	lua_settable(L, -3);
+	lua_pushstring(L, "__index");
+	lua_pushcfunction(L, dummy_index);
+	lua_settable(L, -3);
+	lua_pushstring(L, "__newindex");
+	lua_pushcfunction(L, dummy_newindex);
+	lua_settable(L, -3);
 	lua_setmetatable(L, -2);
+
+	lua_pushstring (L, "userdata");
+	lua_gettable (L, LUA_REGISTRYINDEX);
+	lua_pushlightuserdata (L, object);
+	lua_pushvalue (L, -3);
+	lua_settable (L, -3);
+	lua_pop(L, 1);
     } else {
 	printf ("Could not bind a socket to local "
 		"interface %s at port %d (%s).\n",
