@@ -27,8 +27,14 @@
 
 Node *list;
 
-static void callhooks (lua_State *L, void *key, int reference)
+static void callhooks (lua_State *L, void *key,
+		       int reference, int nargs, ...)
 {
+    va_list ap;
+    int i;
+
+    va_start(ap, nargs);
+
     if (reference != LUA_REFNIL) {
 	lua_rawgeti(L, LUA_REGISTRYINDEX, reference);
 	
@@ -37,8 +43,14 @@ static void callhooks (lua_State *L, void *key, int reference)
 	    lua_pushlightuserdata (L, key);
 	    lua_gettable (L, -2);
 	    lua_replace (L, -2);
+
+	    /* Push extra arguments and call the function. */
 	    
-	    luaX_call(L, 1, 0);
+	    for (i = 0 ; i < nargs ; i += 1) {
+		lua_pushnumber (L, va_arg(ap, lua_Number));
+	    }
+
+	    luaX_call(L, 1 + nargs, 0);
 	} else if (lua_istable (L, -1)) {
 	    int i, n;
 	    
@@ -51,14 +63,23 @@ static void callhooks (lua_State *L, void *key, int reference)
 
 		lua_pushlightuserdata (L, key);
 		lua_gettable (L, -3);
-		luaX_call (L, 1, 0);
 	    }
+
+	    /* Push extra arguments and call the function. */
+	    
+	    for (i = 0 ; i < nargs ; i += 1) {
+		lua_pushnumber (L, va_arg(ap, lua_Number));
+	    }
+
+	    luaX_call(L, 1 + nargs, 0);
 	    
 	    lua_pop (L, 2);
 	} else {
 	    lua_pop (L, 1);
 	}
     }
+
+    va_end(ap);
 }
 
 static int ancestry_tostring(lua_State *L)
@@ -68,6 +89,25 @@ static int ancestry_tostring(lua_State *L)
     return 1;
 }
 
+static int ancestry_len(lua_State *L)
+{
+    id object;
+    int i;
+
+    lua_getmetatable (_L, 1);
+    lua_pushstring (_L, "__node");
+    lua_gettable (_L, -2);
+
+    
+    for (i = 0, object = *(id *)lua_touserdata(L, -1);
+	 object;
+	 i += 1, object = [object parent]);
+
+    lua_pushinteger (L, i);
+
+    return 1;
+}
+
 static int ancestry_index(lua_State *L)
 {
     int i, n;
@@ -129,6 +169,8 @@ static int ancestry_newindex(lua_State *L)
     
     [super init];
     [self add: sizeof (list) / sizeof (char *) Properties: list];
+
+    self->customname = NULL;
     
     self->link = LUA_REFNIL;
     self->unlink = LUA_REFNIL;
@@ -140,11 +182,21 @@ static int ancestry_newindex(lua_State *L)
     self->begin = LUA_REFNIL;
     
     self->tag = 0;
+    self->tagged = 0;
     self->linked = 0;
 
     return self;
 }
 
+-(const char *) name
+{
+    if (self->customname) {
+	return self->customname;
+    } else {
+	return [[self class] name];
+    }
+}
+
 -(int) linked
 {
     return self->linked;
@@ -157,9 +209,9 @@ static int ancestry_newindex(lua_State *L)
     /* Call the hooks first. */
     
     if (!self->linked) {
-	callhooks (_L, self, self->link);
+	callhooks (_L, self, self->link, 0);
     } else {
-	callhooks (_L, self, self->unlink);
+	callhooks (_L, self, self->unlink, 0);
     }
 
     /* Toggle ourselves. */
@@ -179,7 +231,7 @@ static int ancestry_newindex(lua_State *L)
 
     /* Call the node's input hook. */
     
-    callhooks (_L, self, self->input);
+    callhooks (_L, self, self->input, 0);
     
     for(child = [self children] ; child ; child = [child sister]) {
 	[child input];
@@ -190,7 +242,7 @@ static int ancestry_newindex(lua_State *L)
 {
     id child;
     
-    callhooks (_L, self, self->begin);
+    callhooks (_L, self, self->begin, 0);
     
     for(child = [self children] ; child ; child = [child sister]) {
 	[child begin];
@@ -201,7 +253,7 @@ static int ancestry_newindex(lua_State *L)
 {
     id child;
 
-    callhooks (_L, self, self->step);
+    callhooks (_L, self, self->step, 2, h, t);
 
     for(child = [self children] ; child ; child = [child sister]) {
 	[child stepBy: h at: t];
@@ -212,7 +264,7 @@ static int ancestry_newindex(lua_State *L)
 {
     id child;
     
-    callhooks (_L, self, self->prepare);
+    callhooks (_L, self, self->prepare, 0);
     
     for(child = [self children] ; child ; child = [child sister]) {
 	[child prepare];
@@ -223,7 +275,7 @@ static int ancestry_newindex(lua_State *L)
 {
     id child;
 
-    callhooks (_L, self, self->traverse);
+    callhooks (_L, self, self->traverse, 1, pass);
 
     for(child = [self children] ; child ; child = [child sister]) {
 	[child traversePass: pass];
@@ -234,13 +286,27 @@ static int ancestry_newindex(lua_State *L)
 {
     id child;
     
-    callhooks (_L, self, self->finish);
+    callhooks (_L, self, self->finish, 0);
     
     for(child = [self children] ; child ; child = [child sister]) {
 	[child finish];
     }
 }
 
+-(void) free
+{
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->unlink);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->link);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->input);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->step);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->traverse);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->prepare);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->finish);
+    luaL_unref (_L, LUA_REGISTRYINDEX, self->begin);
+
+    [super free];
+}
+
 -(void) get
 {
     const char *k;
@@ -254,6 +320,9 @@ static int ancestry_newindex(lua_State *L)
 	lua_pushstring(_L, "__node");
 	lua_pushvalue (_L, 1);
 	lua_settable(_L, -3);
+	lua_pushstring(_L, "__len");
+	lua_pushcfunction(_L, (lua_CFunction)ancestry_len);
+	lua_settable(_L, -3);
 	lua_pushstring(_L, "__index");
 	lua_pushcfunction(_L, (lua_CFunction)ancestry_index);
 	lua_settable(_L, -3);
@@ -268,8 +337,16 @@ static int ancestry_newindex(lua_State *L)
 	lua_getmetatable (_L, 1);
 	lua_pushstring (_L, "__parent");
 	lua_gettable (_L, -2);
+    } else if (!xstrcmp(k, "name")) {
+	lua_pushstring (_L, [self name]);
     } else if (!xstrcmp(k, "tag")) {
-	 lua_pushinteger (_L, self->tag);
+	if (self->tagged) {
+	    lua_pushinteger (_L, self->tag);
+	} else {
+	    lua_pushboolean (_L, 0);
+	}
+    } else if (!xstrcmp(k, "linked")) {
+	lua_pushboolean (_L, self->linked);
     } else if (!xstrcmp(k, "link")) {
         lua_rawgeti (_L, LUA_REGISTRYINDEX, self->link);
     } else if (!xstrcmp(k, "unlink")) {
@@ -311,8 +388,22 @@ static int ancestry_newindex(lua_State *L)
 		lua_settable (_L, -3);
 	    }
 	}
+    } else if (!xstrcmp(k, "name")) {
+	if (lua_type (_L, 3) == LUA_TSTRING) {
+	    self->customname = realloc(self->customname,
+				       lua_strlen (_L, 3) + 1);
+	    strcpy (self->customname, lua_tostring (_L, 3));
+	} else {
+	    free (self->customname);
+	    self->customname = NULL;
+	}
     } else if (!xstrcmp(k, "tag")) {
-	self->tag = lua_tointeger (_L, 3);
+	if (lua_isnumber (_L, 3)) {
+	    self->tag = lua_tointeger (_L, 3);
+	    self->tagged = 1;
+	} else {
+	    self->tagged = 0;
+	}
     } else if (!xstrcmp(k, "unlink")) {
         luaL_unref (_L, LUA_REGISTRYINDEX, self->unlink);
         self->unlink = luaL_ref (_L, LUA_REGISTRYINDEX);
