@@ -22,7 +22,8 @@
 #include "item.h"
 
 @interface Node: Item {
-    int linked, tag;
+    char *customname;
+    int linked, tag, tagged;
     int link, unlink, input, prepare, step, traverse, finish, begin;
 }
 
