@@ -23,9 +23,6 @@
 #include "system.h"
 #include "polyhedron.h"
 
-extern void addContactJoint (Body *a, Body *b, dVector3 pos, dVector3 normal,
-			     dReal depth, dReal mu, dReal bounce);
-
 static int addforce (lua_State *L)
 {
     Body *object;
@@ -139,7 +136,7 @@ static int addrelativetorque (lua_State *L)
 	    lua_pop(L, 1);
 	}
 	
-	dBodyAddRelTorque ([object body], T[0], T[1], T[2]);\
+	dBodyAddRelTorque ([object body], T[0], T[1], T[2]);
 	dBodyEnable([object body]);
     }
 
@@ -262,7 +259,7 @@ static int rotatemass (lua_State *L)
 
     for(j = 0; j < 3 ; j += 1) {
 	for(i = 0; i < 3 ; i += 1) {
-	    lua_rawgeti(L, 2, 4 * j + i + 1);
+	    lua_rawgeti(L, 2, 3 * j + i + 1);
 	    R[4 * j + i] = lua_tonumber (L, -1);
 	    lua_pop(L, 1);
 	}
@@ -367,14 +364,16 @@ static int spring (lua_State *L)
     return 1;
 }
 
-static int collide (lua_State *L)
+static int contact (lua_State *L)
 {
     dContact contact;
     dJointID j;
     Body *a, *b;
-    dReal mu, depth, bounce;
+    dReal mu, depth, epsilon, sigma, tau;
     dVector3 pos, normal;
-    int i;
+    int i, simple;
+
+    simple = lua_toboolean (L, lua_upvalueindex (1));
 
     luaL_checktype (L, 1, LUA_TUSERDATA);
     luaL_checktype (L, 2, LUA_TUSERDATA);
@@ -382,8 +381,10 @@ static int collide (lua_State *L)
     luaL_checktype (L, 4, LUA_TTABLE);
 
     depth = (dReal)luaL_checknumber (L, 5);
-    mu = (dReal)luaL_checknumber (L, 6);
-    bounce = (dReal)luaL_checknumber (L, 7);
+    mu = (dReal)luaL_optnumber (L, 6, 0);
+    epsilon = (dReal)luaL_optnumber (L, 7, 0);
+    sigma = (dReal)luaL_optnumber (L, 8, 0);
+    tau = (dReal)luaL_optnumber (L, 9, 1);
 
     for(i = 0; i < 3; i += 1) {
 	lua_rawgeti(L, 3, i + 1);
@@ -398,20 +399,34 @@ static int collide (lua_State *L)
     a = *(Body **)lua_touserdata (L, 1);
     b = *(Body **)lua_touserdata (L, 2);
 
-/*     { */
-/* 	dReal *u; */
-
-/* 	u = dBodyGetLinearVel ([a body]); */
-/* 	printf ("v_a = %f, %f, %f\n", u[0], u[1], u[2]); */
-	
-/* 	u = dBodyGetLinearVel ([b body]); */
-/* 	printf ("v_b = %f, %f, %f\n", u[0], u[1], u[2]); */
-/*     } */
+    /* Set surface parameters. */
     
-    contact.surface.mode = dContactBounce | dContactApprox1 | dContactFDir1;
-    contact.surface.mu = mu;
-    contact.surface.bounce = bounce;
-    contact.surface.bounce_vel = 0.01;
+    contact.surface.mode = 0;
+		    
+    if (mu > 0) {
+	if (!simple) {
+	    contact.surface.mode |= dContactApprox1;
+	}
+	
+	contact.surface.mode |= dContactFDir1;
+	contact.surface.mu = mu;
+    }
+
+    if (epsilon > 0) {
+	contact.surface.mode |= dContactBounce;
+	contact.surface.bounce = epsilon;
+	contact.surface.bounce_vel = 0.01;
+    }
+
+    if (sigma > 0) {
+	contact.surface.mode |= dContactSoftCFM;
+	contact.surface.soft_cfm = sigma;
+    }
+
+    if (tau < 1) {
+	contact.surface.mode |= dContactSoftERP;
+	contact.surface.soft_erp = tau;
+    }
 
     for (i = 0 ; i < 3; i += 1) {
 	contact.geom.pos[i] = pos[i];
@@ -420,45 +435,50 @@ static int collide (lua_State *L)
 
     dSafeNormalize3 (contact.geom.normal);
 
-/*     printf ("%f, %f, %f\n", */
-/* 	    contact.geom.normal[0], */
-/* 	    contact.geom.normal[1], */
-/* 	    contact.geom.normal[2]); */
     {
+	dBodyID p, q;
 	dVector3 u, v, delta;
 	dReal *r, *n, ndotdelta;
 
 	r = pos;
 	n = normal;
+	p = [a body];
+	q = [b body];
 		
-	dSetZero (u, 3);
-	dSetZero (v, 3);
+	if (p) {
+	    dBodyGetPointVel (p, r[0], r[1], r[2], u);
+	} else {
+	    dSetZero (u, 3);
+	}
 		
-	if ([a body]) {
-	    dBodyGetPointVel ([a body], r[0], r[1], r[2], u);
-	}
-		    
-	if ([b body]) {
-	    dBodyGetPointVel ([b body], r[0], r[1], r[2], v);
-	}
+	if (q) {
+	    dBodyGetPointVel (q, r[0], r[1], r[2], v);
+	} else {
+	    dSetZero (v, 3);
+	}	
 
-	delta[0] = u[0] - v[0];
-	delta[1] = u[1] - v[1];
-	delta[2] = u[2] - v[2];
-		
+	dOP (delta, -, u, v);		
 	ndotdelta = dDOT(n, delta);
-	contact.fdir1[0] = delta[0] - ndotdelta * n[0]; 
-	contact.fdir1[1] = delta[1] - ndotdelta * n[1]; 
-	contact.fdir1[2] = delta[2] - ndotdelta * n[2];
-    }
+
+	/* Only consider this contact if the
+	   relative velocity points inward. */
+
+	if (simple || ndotdelta <= 0) {
+	    contact.fdir1[0] = delta[0] - ndotdelta * n[0];
+	    contact.fdir1[1] = delta[1] - ndotdelta * n[1];
+	    contact.fdir1[2] = delta[2] - ndotdelta * n[2];
+
+	    dSafeNormalize3(contact.fdir1);
     
-    contact.geom.depth = depth;
-    contact.geom.g1 = [a geom];
-    contact.geom.g2 = [b geom];
+	    contact.geom.depth = depth;
+	    contact.geom.g1 = [a geom];
+	    contact.geom.g2 = [b geom];
 
-    j = dJointCreateContact (_WORLD, _GROUP, &contact);
+	    j = dJointCreateContact (_WORLD, _GROUP, &contact);
 
-    dJointAttach (j, [a body], [b body]);
+	    dJointAttach (j, [a body], [b body]);
+	}
+    }
 
     return 0;
 }
@@ -700,12 +720,21 @@ int luaopen_physics (lua_State *L)
 	{"vectorfrombody", vectorfrombody},
 	{"vectortobody", vectortobody},
 	{"pointvelocity", pointvelocity},
-	{"collide", collide},
 	{"reattach", reattach},
 	{NULL, NULL}
     };
 
     luaL_register (L, "physics", physics);
 
+    lua_pushstring (L, "addcontact");
+    lua_pushboolean (L, 0);
+    lua_pushcclosure (L, contact, 1);
+    lua_settable (L, -3);
+
+    lua_pushstring (L, "addsimplecontact");
+    lua_pushboolean (L, 1);
+    lua_pushcclosure (L, contact, 1);
+    lua_settable (L, -3);
+
     return 0;
 }
