@@ -32,4 +32,26 @@
     return self;
 }
 
+-(void) traversePass: (int)pass
+{
+    if (pass == 2 && self->debug) {
+	glUseProgramObjectARB(0);
+
+	glMatrixMode (GL_MODELVIEW);
+	glPushMatrix();
+	glMultMatrixd ([self matrix]);
+
+	glPointSize (5);
+	glColor3f (0, 1, 0);
+
+	glBegin (GL_POINTS);
+	glVertex3f (0, 0, 0);
+	glEnd();
+
+	glPopMatrix();
+    }
+    
+    [super traversePass: pass];
+}
+
 @end
