@@ -35,8 +35,8 @@
 	glEnable(GL_POINT_SMOOTH);
 	glEnable(GL_BLEND);
     
-	glPointSize ([self width]);
-	glColor4fv([self color]);
+	glPointSize (self->width);
+	glColor4dv(self->color);
 
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glVertexPointer(3, GL_FLOAT, 0, self->vertices);
