@@ -17,9 +17,9 @@
 #ifndef _SHADOW_H_
 #define _SHADOW_H_
 
-#include "node.h"
+#include "transform.h"
 
-@interface Shadow: Node {
+@interface Shadow: Transform {
 }
 @end
 
