@@ -192,7 +192,14 @@
 
 	    /* Resetting the stops makes sure that lo remains
 	       smaller than hi between calls. */
-	
+
+	    /* printf ("%f %f %f %f %f\n", */
+	    /* 	    self->stops[0], */
+	    /* 	    self->stops[1], */
+	    /* 	    self->hardness[0], */
+	    /* 	    self->hardness[1], */
+	    /* 	    self->bounce); */
+
 	    dJointSetSliderParam ([self joint], dParamLoStop, -dInfinity);
 	    dJointSetSliderParam ([self joint], dParamHiStop, dInfinity);
 
@@ -230,85 +237,87 @@
 	a = dJointGetBody ([self joint], 0);
 	b = dJointGetBody ([self joint], 1);
 
-	assert (a || b);
+	if (a || b) {
+	    l = dJointGetSliderPosition([self joint]);
+	    dJointGetSliderAxis ([self joint], x);
 
-	l = dJointGetSliderPosition([self joint]);
-	dJointGetSliderAxis ([self joint], x);
-
-	if (!a) {
-	    p = dBodyGetPosition (b);
-	    l = -l;
-	    n = -l;
-	} else if (!b) {
-	    p = dBodyGetPosition (a);
-	    n = -l;
-	} else {
-	    p = dBodyGetPosition (a);	
-	    q = dBodyGetPosition (b);
+	    if (!a) {
+		p = dBodyGetPosition (b);
+		l = -l;
+		n = -l;
+	    } else if (!b) {
+		p = dBodyGetPosition (a);
+		n = -l;
+	    } else {
+		p = dBodyGetPosition (a);	
+		q = dBodyGetPosition (b);
 	    
-	    n = (q[0] - p[0]) * x[0] +
-		(q[1] - p[1]) * x[1] +
-		(q[2] - p[2]) * x[2];
-	}
+		n = (q[0] - p[0]) * x[0] +
+		    (q[1] - p[1]) * x[1] +
+		    (q[2] - p[2]) * x[2];
+	    }
 		
-	glUseProgramObjectARB(0);
+	    glUseProgramObjectARB(0);
 
-	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	glEnable(GL_DEPTH_TEST);
-	glEnable(GL_LINE_SMOOTH);
-	glEnable(GL_POINT_SMOOTH);
-	glEnable(GL_BLEND);
-	glDepthMask (GL_FALSE);
+	    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	    glEnable(GL_DEPTH_TEST);
+	    glEnable(GL_LINE_SMOOTH);
+	    glEnable(GL_POINT_SMOOTH);
+	    glEnable(GL_BLEND);
+	    glDepthMask (GL_FALSE);
 
-	/* Draw the outer shell. */
+	    /* Draw the outer shell. */
 	
-	glColor3f(0, 1, 0);
-	glLineWidth (3);
+	    glColor3f(0, 1, 0);
+	    glLineWidth (2);
 
-	glBegin (GL_LINES);
-	glVertex3f (p[0], p[1], p[2]);
-	glVertex3f (p[0] + x[0] * (n + l),
-		    p[1] + x[1] * (n + l),
-		    p[2] + x[2] * (n + l));
-	glEnd();
+	    glBegin (GL_LINES);
+	    glVertex3f (p[0], p[1], p[2]);
+	    glVertex3f (p[0] + x[0] * (n + l),
+			p[1] + x[1] * (n + l),
+			p[2] + x[2] * (n + l));
+	    glEnd();
 
-	/* Draw the shaft. */
+	    /* Draw the shaft. */
 	
-	glColor3f(1, 1, 0);
-	glLineWidth (1);
+	    glColor3f(1, 1, 0);
+	    glLineWidth (1);
 	
-	glBegin (GL_LINES);
-	glVertex3f (p[0] + x[0] * (n + l),
-		    p[1] + x[1] * (n + l),
-		    p[2] + x[2] * (n + l));
-	glVertex3f (p[0] + x[0] * n,
-		    p[1] + x[1] * n,
-		    p[2] + x[2] * n);
-	glEnd();
+	    glBegin (GL_LINES);
+	    glVertex3f (p[0] + x[0] * (n + l),
+			p[1] + x[1] * (n + l),
+			p[2] + x[2] * (n + l));
+	    glVertex3f (p[0] + x[0] * n,
+			p[1] + x[1] * n,
+			p[2] + x[2] * n);
+	    glEnd();
 	
-	/* Draw the axis. */
+	    /* Draw the axis. */
 	
-	glColor3f(0, 0, 1);
-	glLineWidth (1);
+	    glColor3f(0, 0, 1);
+	    glLineWidth (1);
 	
-	glBegin (GL_LINES);
-	glVertex3f (p[0], p[1], p[2]);
-	glVertex3f (p[0] + x[0],
-		    p[1] + x[1],
-		    p[2] + x[2]);
-	glEnd();
+	    glBegin (GL_LINES);
+	    glVertex3f (p[0], p[1], p[2]);
+	    glVertex3f (p[0] + x[0],
+			p[1] + x[1],
+			p[2] + x[2]);
+	    glEnd();
 
-	glBegin (GL_POINTS);
-	glVertex3f (p[0] + x[0],
-		    p[1] + x[1],
-		    p[2] + x[2]);
-	glEnd();
+	    glPointSize (3);
 
-	glDepthMask (GL_TRUE);
-	glDisable(GL_BLEND);
-	glDisable(GL_LINE_SMOOTH);
-	glDisable(GL_POINT_SMOOTH);
-	glDisable(GL_DEPTH_TEST);
+	    glBegin (GL_POINTS);
+	    glVertex3f (p[0] + x[0],
+			p[1] + x[1],
+			p[2] + x[2]);
+	    glEnd();
+
+	    glDepthMask (GL_TRUE);
+	    glDisable(GL_BLEND);
+	    glDisable(GL_LINE_SMOOTH);
+	    glDisable(GL_POINT_SMOOTH);
+	    glDisable(GL_DEPTH_TEST);
+	}
     }
     
     [super traversePass: pass];
