@@ -19,24 +19,18 @@
 
 #include <lua.h>
 #include <AL/al.h>
-#include "node.h"
+#include "transform.h"
 
-@interface Source: Node {
+@interface Source: Transform {
     ALuint index;
     ALenum state;
-    ALuint name, frequency;
 
-    double tick;
-    int size, finished;
+    int finished;
+    double delta;
+    float velocity[3];
 }
 
 -(ALint) index;
--(Source *) initWithSamples: (ALshort *) samples
-                     ofSize: (int) n
-		atFrequency: (int) f;
--(void) freeBuffers;
--(int) length;
--(int) frequency;
 
 @end
 
