@@ -18,16 +18,14 @@
 #define _STATIC_H_
 
 #include <lua.h>
-#include "geometry.h"
+#include "transform.h"
 
-@interface Static: Geometry {
+@interface Static: Transform {
     GLfloat *vertices;
     GLushort *indices;
     
     GLuint buffers[2];
     int size[2];
-
-    float bound;
 }
 
 -(Static *) initWithVertices: (GLfloat *)floats
@@ -36,7 +34,6 @@
 -(void) freeBuffers;
 -(int) vertices;
 -(int) indices;
--(float) bound;
 
 @end
 
