@@ -87,6 +87,13 @@ static void calltimedhooks (lua_State *L, void *key, int reference,
     }
 }
 
+-(void) free
+{
+     luaL_unref (_L, LUA_REGISTRYINDEX, self->tick);
+
+     [super free];
+}
+
 -(void) tick
 {
     struct timespec time;
