@@ -30,9 +30,7 @@
     [self add: sizeof (list) / sizeof (char *) Properties: list];
 
     self->state = 0;
-    self->on = LUA_REFNIL;
-    self->off = LUA_REFNIL;
-    
+
     return self;
 }
 
@@ -45,9 +43,17 @@
     if (!xstrcmp(k, "state")) {
 	lua_pushboolean (_L, self->state);
     } else if (!xstrcmp(k, "on")) {
-	lua_rawgeti(_L, LUA_REGISTRYINDEX, self->on);
+	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__on");
+	lua_replace (_L, 2);
+        lua_rawget (_L, 1);	
     } else if (!xstrcmp(k, "off")) {
-	lua_rawgeti(_L, LUA_REGISTRYINDEX, self->off);
+	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__off");
+	lua_replace (_L, 2);
+        lua_rawget (_L, 1);	
     } else {
 	[super get];
     }
@@ -67,19 +73,29 @@
 
 	    if (self->state) {
 		/* Unlink off. */
+
+		lua_getmetatable (_L, 1);
+		lua_pushstring (_L, "__off");
+		lua_gettable (_L, -2);
+		lua_replace (_L, -2);
 		
-		if (self->off != LUA_REFNIL) {    
-		    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->off);
+		if (!lua_isnil (_L, -1)) {    
 		    lua_pushliteral (_L, "parent");
 		    lua_pushnil (_L);
 		    lua_settable (_L, -3);
 		    lua_pop (_L, 1);
-		}
+		} else {
+		    lua_pop (_L, 1);
+		}   
 	
 		/* Link on. */
+
+		lua_getmetatable (_L, 1);
+		lua_pushstring (_L, "__on");
+		lua_gettable (_L, -2);
+		lua_replace (_L, -2);
 		
-		if (self->on != LUA_REFNIL) {    
-		    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->on);
+		if (!lua_isnil (_L, -1)) {    
 		    lua_pushliteral (_L, "parent");
 
 		    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
@@ -89,22 +105,34 @@
 
 		    lua_settable (_L, -3);
 		    lua_pop (_L, 1);
-		}
+		} else {
+		    lua_pop (_L, 1);
+		}  
 	    } else {
 		/* Unlink on. */
+
+		lua_getmetatable (_L, 1);
+		lua_pushstring (_L, "__on");
+		lua_gettable (_L, -2);
+		lua_replace (_L, -2);
 		
-		if (self->on != LUA_REFNIL) {    
-		    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->on);
+		if (!lua_isnil (_L, -1)) {    
 		    lua_pushliteral (_L, "parent");
 		    lua_pushnil (_L);
 		    lua_settable (_L, -3);
 		    lua_pop (_L, 1);
-		}
+		} else {
+		    lua_pop (_L, 1);
+		} 
 		
 		/* Link off. */
+
+		lua_getmetatable (_L, 1);
+		lua_pushstring (_L, "__off");
+		lua_gettable (_L, -2);
+		lua_replace (_L, -2);
 		
-		if (!self->state && self->off != LUA_REFNIL) {    
-		    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->off);
+		if (!lua_isnil (_L, -1)) {    
 		    lua_pushliteral (_L, "parent");
 
 		    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
@@ -114,65 +142,89 @@
 
 		    lua_settable (_L, -3);
 		    lua_pop (_L, 1);
-		}
+		} else {
+		    lua_pop (_L, 1);
+		} 
 	    }
 	}
     } else if (!xstrcmp(k, "on")) {
-	/* Make sure we're unlinked. */
+    	/* Make sure we're unlinked. */
+
+	lua_getmetatable (_L, 1);
+	lua_pushstring (_L, "__on");
+	lua_gettable (_L, -2);
+	lua_replace (_L, -2);
 	
-	if (self->on != LUA_REFNIL) {    
-	    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->on);
-	    lua_pushliteral (_L, "parent");
-	    lua_pushnil (_L);
-	    lua_settable (_L, -3);
+    	if (!lua_isnil (_L, -1)) {
+    	    lua_pushliteral (_L, "parent");
+    	    lua_pushnil (_L);
+    	    lua_settable (_L, -3);
+    	    lua_pop (_L, 1);
+    	} else {
 	    lua_pop (_L, 1);
 	}
+
+    	/* Link if needed. */
 	
-	luaL_unref (_L, LUA_REGISTRYINDEX, self->on);
-	self->on = luaL_ref (_L, LUA_REGISTRYINDEX);
+    	if (self->state && !lua_isnil (_L, 3)) {
+	    lua_pushvalue (_L, 3);
+    	    lua_pushliteral (_L, "parent");
 
-	/* Link if needed. */
+    	    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
+    	    lua_pushlightuserdata (_L, self);
+    	    lua_gettable (_L, -2);
+    	    lua_replace (_L, -2);
+
+    	    lua_settable (_L, -3);
+    	    lua_pop (_L, 1);
+    	}
+
+	/* Save a reference. */
 	
-	if (!self->state && self->on != LUA_REFNIL) {    
-	    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->on);
-	    lua_pushliteral (_L, "parent");
-
-	    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
-	    lua_pushlightuserdata (_L, self);
-	    lua_gettable (_L, -2);
-	    lua_replace (_L, -2);
-
-	    lua_settable (_L, -3);
-	    lua_pop (_L, 1);
-	}
+ 	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__on");
+	lua_replace (_L, 2);
+        lua_rawset (_L, 1);	
     } else if (!xstrcmp(k, "off")) {
-	/* Make sure we're unlinked. */
+    	/* Make sure we're unlinked. */
+
+	lua_getmetatable (_L, 1);
+	lua_pushstring (_L, "__off");
+	lua_gettable (_L, -2);
+	lua_replace (_L, -2);
 	
-	if (self->off != LUA_REFNIL) {    
-	    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->off);
-	    lua_pushliteral (_L, "parent");
-	    lua_pushnil (_L);
-	    lua_settable (_L, -3);
+    	if (!lua_isnil (_L, -1)) {
+    	    lua_pushliteral (_L, "parent");
+    	    lua_pushnil (_L);
+    	    lua_settable (_L, -3);
+    	    lua_pop (_L, 1);
+    	} else {
 	    lua_pop (_L, 1);
 	}
+
+    	/* Link if needed. */
 	
-	luaL_unref (_L, LUA_REGISTRYINDEX, self->off);
-	self->off = luaL_ref (_L, LUA_REGISTRYINDEX);
+    	if (!self->state && !lua_isnil (_L, 3)) {
+	    lua_pushvalue (_L, 3);
+    	    lua_pushliteral (_L, "parent");
 
-	/* Link if needed. */
+    	    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
+    	    lua_pushlightuserdata (_L, self);
+    	    lua_gettable (_L, -2);
+    	    lua_replace (_L, -2);
+
+    	    lua_settable (_L, -3);
+    	    lua_pop (_L, 1);
+    	}
+
+	/* Save a reference. */
 	
-	if (!self->state && self->off != LUA_REFNIL) {    
-	    lua_rawgeti (_L, LUA_REGISTRYINDEX, self->off);
-	    lua_pushliteral (_L, "parent");
-
-	    lua_getfield (_L, LUA_REGISTRYINDEX, "userdata");
-	    lua_pushlightuserdata (_L, self);
-	    lua_gettable (_L, -2);
-	    lua_replace (_L, -2);
-
-	    lua_settable (_L, -3);
-	    lua_pop (_L, 1);
-	}
+ 	lua_getmetatable (_L, 1);
+	lua_replace (_L, 1);
+	lua_pushstring (_L, "__off");
+	lua_replace (_L, 2);
+        lua_rawset (_L, 1);	
     } else {
 	[super set];
     }
