@@ -20,7 +20,7 @@
 #include "node.h"
 
 @interface Toggle: Node {
-    int state, on, off;
+    int state;
 }
 @end
 
